import React, { createContext, useContext, useEffect, Component, useState } from 'react';

function defaultEqualityCheck(a, b) {
  return a === b;
}

function areArgumentsShallowlyEqual(equalityCheck, prev, next) {
  if (prev === null || next === null || prev.length !== next.length) {
    return false;
  }

  // Do this in a for loop (and not a `forEach` or an `every`) so we can determine equality as fast as possible.
  var length = prev.length;
  for (var i = 0; i < length; i++) {
    if (!equalityCheck(prev[i], next[i])) {
      return false;
    }
  }

  return true;
}

function defaultMemoize(func) {
  var equalityCheck = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : defaultEqualityCheck;

  var lastArgs = null;
  var lastResult = null;
  // we reference arguments instead of spreading them for performance reasons
  return function () {
    if (!areArgumentsShallowlyEqual(equalityCheck, lastArgs, arguments)) {
      // apply arguments instead of spreading for performance.
      lastResult = func.apply(null, arguments);
    }

    lastArgs = arguments;
    return lastResult;
  };
}

function getDependencies(funcs) {
  var dependencies = Array.isArray(funcs[0]) ? funcs[0] : funcs;

  if (!dependencies.every(function (dep) {
    return typeof dep === 'function';
  })) {
    var dependencyTypes = dependencies.map(function (dep) {
      return typeof dep;
    }).join(', ');
    throw new Error('Selector creators expect all input-selectors to be functions, ' + ('instead received the following types: [' + dependencyTypes + ']'));
  }

  return dependencies;
}

function createSelectorCreator(memoize) {
  for (var _len = arguments.length, memoizeOptions = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    memoizeOptions[_key - 1] = arguments[_key];
  }

  return function () {
    for (var _len2 = arguments.length, funcs = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      funcs[_key2] = arguments[_key2];
    }

    var recomputations = 0;
    var resultFunc = funcs.pop();
    var dependencies = getDependencies(funcs);

    var memoizedResultFunc = memoize.apply(undefined, [function () {
      recomputations++;
      // apply arguments instead of spreading for performance.
      return resultFunc.apply(null, arguments);
    }].concat(memoizeOptions));

    // If a selector is called with the exact same arguments we don't need to traverse our dependencies again.
    var selector = memoize(function () {
      var params = [];
      var length = dependencies.length;

      for (var i = 0; i < length; i++) {
        // apply arguments instead of spreading and mutate a local list of params for performance.
        params.push(dependencies[i].apply(null, arguments));
      }

      // apply arguments instead of spreading for performance.
      return memoizedResultFunc.apply(null, params);
    });

    selector.resultFunc = resultFunc;
    selector.dependencies = dependencies;
    selector.recomputations = function () {
      return recomputations;
    };
    selector.resetRecomputations = function () {
      return recomputations = 0;
    };
    return selector;
  };
}

var createSelector = createSelectorCreator(defaultMemoize);

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = function(d, b) {
    extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return extendStatics(d, b);
};

function __extends(d, b) {
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
}

var getLocalizedElement = function (options) {
    var translation = options.translation, data = options.data, renderInnerHtml = options.renderInnerHtml;
    var translatedValueOrArray = templater(translation, data);
    // if result of templater is string, do the usual stuff
    if (typeof translatedValueOrArray === 'string') {
        return renderInnerHtml === true && hasHtmlTags(translatedValueOrArray)
            ? React.createElement('span', {
                dangerouslySetInnerHTML: { __html: translatedValueOrArray }
            })
            : translatedValueOrArray;
    }
    // at this point we know we have react components;
    // check if there are HTMLTags in the translation (not allowed)
    for (var _i = 0, translatedValueOrArray_1 = translatedValueOrArray; _i < translatedValueOrArray_1.length; _i++) {
        var portion = translatedValueOrArray_1[_i];
        if (typeof portion === 'string' && hasHtmlTags(portion)) {
            warning('HTML tags in the translation string are not supported when passing React components as arguments to the translation.');
            return '';
        }
    }
    // return as Element
    return React.createElement.apply(React, ['span', null].concat(translatedValueOrArray));
};
var hasHtmlTags = function (value) {
    var pattern = /(&[^\s]*;|<\/?\w+((\s+\w+(\s*=\s*(?:".*?"|'.*?'|[\^'">\s]+))?)+\s*|\s*)\/?>)/;
    return value.search(pattern) >= 0;
};
/**
 * @func templater
 * @desc A poor mans template parser
 * @param {string} strings The template string
 * @param {object} data The data that should be inserted in template
 * @return {string} The template string with the data merged in
 */
var templater = function (strings, data) {
    if (data === void 0) { data = {}; }
    if (!strings)
        return '';
    // ${**}
    // brackets to include it in the result of .split()
    var genericPlaceholderPattern = '(\\${\\s*[^\\s}]+\\s*})';
    // split: from 'Hey ${name}' -> ['Hey', '${name}']
    // filter: clean empty strings
    // map: replace ${prop} with data[prop]
    var splitStrings = strings
        .split(new RegExp(genericPlaceholderPattern, 'gmi'))
        .filter(function (str) { return !!str; })
        .map(function (templatePortion) {
        var matched;
        for (var prop in data) {
            if (matched)
                break;
            var pattern = '\\${\\s*' + prop + '\\s*}';
            var regex = new RegExp(pattern, 'gmi');
            if (regex.test(templatePortion))
                matched = data[prop];
        }
        if (typeof matched === 'undefined')
            return templatePortion;
        return matched;
    });
    // if there is a React element, return as array
    if (splitStrings.some(function (portion) { return React.isValidElement(portion); })) {
        return splitStrings;
    }
    // otherwise concatenate all portions into the translated value
    return splitStrings.reduce(function (translated, portion) {
        return translated + ("" + portion);
    }, '');
};
/**
 * Given an array of Languages return the index of the language
 * that matches the provided langauge code.
 * @param code
 * @param languages
 */
var getIndexForLanguageCode = function (code, languages) {
    return languages.map(function (language) { return language.code; }).indexOf(code);
};
var objectValuesToString = function (data) {
    return !Object.values
        ? Object.keys(data)
            .map(function (key) { return data[key].toString(); })
            .toString()
        : Object.values(data).toString();
};
var validateOptions = function (options) {
    if (options.onMissingTranslation !== undefined &&
        typeof options.onMissingTranslation !== 'function') {
        throw new Error('react-localize-redux: an invalid onMissingTranslation function was provided.');
    }
    if (options.renderToStaticMarkup !== false &&
        typeof options.renderToStaticMarkup !== 'function') {
        throw new Error("\n      react-localize-redux: initialize option renderToStaticMarkup is invalid.\n      Please see https://ryandrewjohnson.github.io/react-localize-redux-docs/#initialize.\n    ");
    }
    return options;
};
/**
 * Return an object with only the translations for the provided language.
 * @param language - The language to match
 * @param languages - Array of all languages
 * @param translations - Translation data for all languages
 */
var getTranslationsForLanguage = function (language, languages, translations) {
    // no language! return no translations
    if (!language) {
        return {};
    }
    var languageCode = language.code;
    var languageIndex = getIndexForLanguageCode(languageCode, languages);
    var keys = Object.keys(translations);
    var totalKeys = keys.length;
    var translationsForLanguage = {};
    for (var i = 0; i < totalKeys; i++) {
        var key = keys[i];
        translationsForLanguage[key] = translations[key][languageIndex];
    }
    return translationsForLanguage;
};
var storeDidChange = function (store, onChange) {
    var currentState;
    function handleChange() {
        var nextState = store.getState();
        if (nextState !== currentState) {
            onChange(currentState);
            currentState = nextState;
        }
    }
    var unsubscribe = store.subscribe(handleChange);
    handleChange();
    return unsubscribe;
};
var getSingleToMultilanguageTranslation = function (language, languageCodes, flattenedTranslations, existingTranslations) {
    var languageIndex = languageCodes.indexOf(language);
    var translations = languageIndex >= 0 ? flattenedTranslations : {};
    var keys = Object.keys(translations);
    var totalKeys = keys.length;
    var singleLanguageTranslations = {};
    var _loop_1 = function (i) {
        var key = keys[i];
        // loop through each language, and for languages that don't match languageIndex
        // keep existing translation data, and for languageIndex store new translation data
        var translationValues = languageCodes.map(function (code, index) {
            var existingValues = existingTranslations[key] || [];
            return index === languageIndex
                ? flattenedTranslations[key]
                : existingValues[index];
        });
        singleLanguageTranslations[key] = translationValues;
    };
    for (var i = 0; i < totalKeys; i++) {
        _loop_1(i);
    }
    return singleLanguageTranslations;
};
// Thanks react-redux for utility function
// https://github.com/reactjs/react-redux/blob/master/src/utils/warning.js
var warning = function (message) {
    if (typeof console !== 'undefined' && typeof console.error === 'function') {
        console.error(message);
    }
    try {
        // This error was thrown as a convenience so that if you enable
        // "break on all exceptions" in your console,
        // it would pause the execution at this line.
        throw new Error(message);
    }
    catch (e) { }
};
var isEmpty = function (value) {
    return value === undefined || value === null || value === '';
};

var flatten = require('flat').flatten;
/**
 * ACTIONS
 */
var INITIALIZE = '@@localize/INITIALIZE';
var ADD_TRANSLATION = '@@localize/ADD_TRANSLATION';
var ADD_TRANSLATION_FOR_LANGUAGE = '@@localize/ADD_TRANSLATION_FOR_LANGUAGE';
var SET_ACTIVE_LANGUAGE = '@@localize/SET_ACTIVE_LANGUAGE';
/**
 * REDUCERS
 */
function languages(state, action) {
    if (state === void 0) { state = []; }
    switch (action.type) {
        case INITIALIZE:
            var isOptionsDefined_1 = action.payload.options !== undefined;
            var actionLanguages = action.payload.languages;
            return actionLanguages.map(function (language, index) {
                var isActive = function (code) {
                    return isOptionsDefined_1 &&
                        action.payload.options.defaultLanguage !== undefined
                        ? code === action.payload.options.defaultLanguage
                        : index === 0;
                };
                // check if it's using array of Language objects, or array of language codes
                return typeof language === 'string'
                    ? { code: language, active: isActive(language) } // language codes
                    : __assign({}, language, { active: isActive(language.code) }); // language objects
            });
        case SET_ACTIVE_LANGUAGE:
            return state.map(function (language) {
                return language.code ===
                    action.payload.languageCode
                    ? __assign({}, language, { active: true }) : __assign({}, language, { active: false });
            });
        default:
            return state;
    }
}
function translations(state, action) {
    if (state === void 0) { state = {}; }
    var flattenedTranslations;
    var translationWithTransform;
    switch (action.type) {
        case INITIALIZE:
            if (!action.payload.translation) {
                return state;
            }
            flattenedTranslations = flatten(action.payload.translation, {
                safe: true
            });
            var isOptionsDefined = action.payload.options !== undefined;
            var firstLanguage = typeof action.payload.languages[0] === 'string'
                ? action.payload.languages[0]
                : action.payload.languages[0].code;
            var defaultLanguage = isOptionsDefined
                ? action.payload.options.defaultLanguage
                : firstLanguage;
            var isMultiLanguageTranslation = Object.keys(flattenedTranslations).some(function (item) { return Array.isArray(flattenedTranslations[item]); });
            // add translation based on whether it is single vs multi language translation data
            var newTranslation = isMultiLanguageTranslation
                ? flattenedTranslations
                : getSingleToMultilanguageTranslation(defaultLanguage, action.languageCodes, flattenedTranslations, state);
            return __assign({}, state, newTranslation);
        case ADD_TRANSLATION:
            translationWithTransform =
                action.payload.translationOptions &&
                    action.payload.translationOptions.translationTransform !== undefined
                    ? action.payload.translationOptions.translationTransform(action.payload.translation || {}, action.languageCodes)
                    : action.payload.translation;
            return __assign({}, state, flatten(translationWithTransform, { safe: true }));
        case ADD_TRANSLATION_FOR_LANGUAGE:
            flattenedTranslations = flatten(action.payload.translation, {
                safe: true
            });
            return __assign({}, state, getSingleToMultilanguageTranslation(action.payload.language, action.languageCodes, flattenedTranslations, state));
        default:
            return state;
    }
}
function options(state, action) {
    if (state === void 0) { state = defaultTranslateOptions; }
    switch (action.type) {
        case INITIALIZE:
            var isDefaultLanguageDefined = action.payload.options !== undefined &&
                action.payload.options.defaultLanguage;
            var defaultLanguage = isDefaultLanguageDefined
                ? action.payload.options.defaultLanguage
                : action.languageCodes[0];
            return __assign({}, state, validateOptions(action.payload.options), { defaultLanguage: defaultLanguage });
        default:
            return state;
    }
}
var defaultTranslateOptions = {
    renderToStaticMarkup: false,
    renderInnerHtml: false,
    ignoreTranslateChildren: false,
    defaultLanguage: '',
    onMissingTranslation: function (_a) {
        var translationId = _a.translationId, languageCode = _a.languageCode;
        return 'Missing translationId: ${ translationId } for language: ${ languageCode }';
    }
};
var initialState = {
    languages: [],
    translations: {},
    options: defaultTranslateOptions
};
var localizeReducer = function (state, action) {
    if (state === void 0) { state = initialState; }
    // execute the languages reducer first as we need access to those values for other reducers
    var languagesState = languages(state.languages, action);
    var languageCodes = languagesState.map(function (language) { return language.code; });
    return {
        languages: languagesState,
        translations: translations(state.translations, __assign({}, action, { languageCodes: languageCodes })),
        options: options(state.options, __assign({}, action, { languageCodes: languageCodes }))
    };
};
/**
 * ACTION CREATORS
 */
var initialize = function (payload) { return ({
    type: INITIALIZE,
    payload: payload
}); };
var addTranslation = function (translation, options) { return ({
    type: ADD_TRANSLATION,
    payload: {
        translation: translation,
        translationOptions: options
    }
}); };
var addTranslationForLanguage = function (translation, language) { return ({
    type: ADD_TRANSLATION_FOR_LANGUAGE,
    payload: { translation: translation, language: language }
}); };
var setActiveLanguage = function (languageCode) { return ({
    type: SET_ACTIVE_LANGUAGE,
    payload: { languageCode: languageCode }
}); };
/**
 * SELECTORS
 */
var getTranslations = function (state) {
    return state.translations;
};
var getLanguages = function (state) {
    return state.languages;
};
var getOptions = function (state) {
    return state.options;
};
var getActiveLanguage = function (state) {
    var languages = getLanguages(state);
    return languages.filter(function (language) { return language.active === true; })[0];
};
/**
 * A custom equality checker that checker that compares an objects keys and values instead of === comparison
 * e.g. {name: 'Ted', sport: 'hockey'} would result in 'name,sport - Ted,hockey' which would be used for comparison
 *
 * NOTE: This works with activeLanguage, languages, and translations data types.
 * If a new data type is added to selector this would need to be updated to accomodate
 */
var translationsEqualSelector = createSelectorCreator(defaultMemoize, function (prev, cur) {
    var prevKeys = typeof prev === 'object' ? Object.keys(prev).toString() : undefined;
    var curKeys = typeof cur === 'object' ? Object.keys(cur).toString() : undefined;
    var prevValues = typeof prev === 'object' ? objectValuesToString(prev) : undefined;
    var curValues = typeof cur === 'object' ? objectValuesToString(cur) : undefined;
    var prevCacheValue = prevKeys !== undefined && prevValues !== undefined
        ? prevKeys + " - " + prevValues
        : prev;
    var curCacheValue = curKeys !== undefined && curValues !== undefined
        ? curKeys + " - " + curValues
        : cur;
    return prevCacheValue === curCacheValue;
});
var getTranslationsForActiveLanguage = translationsEqualSelector(getActiveLanguage, getLanguages, getTranslations, getTranslationsForLanguage);
var getTranslationsForSpecificLanguage = translationsEqualSelector(getLanguages, getTranslations, function (languages, translations) {
    return defaultMemoize(function (languageCode) {
        return getTranslationsForLanguage({ code: languageCode, active: false }, languages, translations);
    });
});
var getTranslate = createSelector(getTranslationsForActiveLanguage, getTranslationsForSpecificLanguage, getActiveLanguage, getOptions, function (translationsForActiveLanguage, getTranslationsForLanguage, activeLanguage, initializeOptions) {
    return function (value, data, translateOptions) {
        if (data === void 0) { data = {}; }
        if (translateOptions === void 0) { translateOptions = {}; }
        var defaultLanguage = initializeOptions.defaultLanguage, defaultOptions = __rest(initializeOptions, ["defaultLanguage"]);
        var overrideLanguage = translateOptions.language;
        var translations = overrideLanguage !== undefined
            ? getTranslationsForLanguage(overrideLanguage)
            : translationsForActiveLanguage;
        var defaultTranslations = activeLanguage && activeLanguage.code === defaultLanguage
            ? translationsForActiveLanguage
            : getTranslationsForLanguage(defaultLanguage);
        var languageCode = overrideLanguage !== undefined
            ? overrideLanguage
            : activeLanguage && activeLanguage.code;
        var mergedOptions = __assign({}, defaultOptions, translateOptions);
        var getTranslation = function (translationId) {
            var hasValidTranslation = translations[translationId] !== undefined;
            var hasValidDefaultTranslation = defaultTranslations[translationId] !== undefined;
            var defaultTranslation = hasValidDefaultTranslation
                ? getLocalizedElement({
                    translation: defaultTranslations[translationId],
                    data: data,
                    renderInnerHtml: mergedOptions.renderInnerHtml
                })
                : "No default translation found! Ensure you've added translations for your default langauge.";
            // if translation is not valid then generate the on missing translation message in it's place
            var translation = hasValidTranslation
                ? translations[translationId]
                : mergedOptions.onMissingTranslation({
                    translationId: translationId,
                    languageCode: languageCode,
                    defaultTranslation: defaultTranslation
                });
            // if translations are missing than ovrride data to include translationId, languageCode
            // as these will be needed to render missing translations message
            var translationData = hasValidTranslation
                ? data
                : { translationId: translationId, languageCode: languageCode };
            return getLocalizedElement({
                translation: translation,
                data: translationData,
                languageCode: languageCode,
                renderInnerHtml: mergedOptions.renderInnerHtml
            });
        };
        if (typeof value === 'string') {
            return getTranslation(value);
        }
        else if (Array.isArray(value)) {
            return value.reduce(function (prev, cur) {
                var _a;
                return __assign({}, prev, (_a = {}, _a[cur] = getTranslation(cur), _a));
            }, {});
        }
        else {
            throw new Error('react-localize-redux: Invalid key passed to getTranslate.');
        }
    };
});

var dispatchInitialize = function (dispatch) { return function (payload) {
    return dispatch(initialize(payload));
}; };
var dispatchAddTranslation = function (dispatch) { return function (translation) {
    return dispatch(addTranslation(translation));
}; };
var dispatchAddTranslationForLanguage = function (dispatch) { return function (translation, language) {
    return dispatch(addTranslationForLanguage(translation, language));
}; };
var dispatchSetActiveLanguage = function (dispatch) { return function (languageCode) {
    return dispatch(setActiveLanguage(languageCode));
}; };
var getContextPropsFromState = function (dispatch) {
    return createSelector(getTranslate, getLanguages, getActiveLanguage, getOptions, function (translate, languages, activeLanguage, options) {
        var defaultLanguage = options.defaultLanguage || (languages[0] && languages[0].code);
        var renderToStaticMarkup = options.renderToStaticMarkup;
        var ignoreTranslateChildren = options.ignoreTranslateChildren !== undefined
            ? options.ignoreTranslateChildren
            : defaultTranslateOptions.ignoreTranslateChildren;
        return {
            translate: translate,
            languages: languages,
            defaultLanguage: defaultLanguage,
            activeLanguage: activeLanguage,
            initialize: dispatchInitialize(dispatch),
            addTranslation: dispatchAddTranslation(dispatch),
            addTranslationForLanguage: dispatchAddTranslationForLanguage(dispatch),
            setActiveLanguage: dispatchSetActiveLanguage(dispatch),
            renderToStaticMarkup: renderToStaticMarkup,
            ignoreTranslateChildren: ignoreTranslateChildren
        };
    });
};
var defaultLocalizeState = localizeReducer(undefined, {});
var defaultContext = getContextPropsFromState(function () { })(defaultLocalizeState);
var LocalizeContext = createContext(defaultContext);

var Translate = function (props) {
    var context = useContext(LocalizeContext);
    var _a = props.id, id = _a === void 0 ? '' : _a, options = props.options, data = props.data;
    var defaultLanguage = options !== undefined ? options.language : context.defaultLanguage;
    var hasFunctionAsChild = typeof props.children === 'function';
    var ignoreTranslateChildren = options !== undefined && !isEmpty(options.ignoreTranslateChildren)
        ? options.ignoreTranslateChildren
        : context.ignoreTranslateChildren;
    var isValidDefaultTranslation = !isEmpty(props.children) && !isEmpty(id) && !isEmpty(defaultLanguage);
    useEffect(function () {
        var _a;
        var shouldAddDefaultTranslation = isValidDefaultTranslation &&
            !hasFunctionAsChild &&
            !ignoreTranslateChildren;
        if (shouldAddDefaultTranslation) {
            var translation = typeof context.renderToStaticMarkup === 'boolean'
                ? props.children
                : context.renderToStaticMarkup(props.children);
            context.addTranslationForLanguage &&
                context.addTranslationForLanguage((_a = {},
                    _a[id] = translation,
                    _a), defaultLanguage);
        }
    }, [id, defaultLanguage]);
    return typeof props.children === 'function'
        ? props.children(context)
        : context.translate && context.translate(id, data, options);
};

function unwrapExports (x) {
	return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
}

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

var reactIs_production_min = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports,"__esModule",{value:!0});
var b="function"===typeof Symbol&&Symbol.for,c=b?Symbol.for("react.element"):60103,d=b?Symbol.for("react.portal"):60106,e=b?Symbol.for("react.fragment"):60107,f=b?Symbol.for("react.strict_mode"):60108,g=b?Symbol.for("react.profiler"):60114,h=b?Symbol.for("react.provider"):60109,k=b?Symbol.for("react.context"):60110,l=b?Symbol.for("react.async_mode"):60111,m=b?Symbol.for("react.concurrent_mode"):60111,n=b?Symbol.for("react.forward_ref"):60112,p=b?Symbol.for("react.suspense"):60113,q=b?Symbol.for("react.memo"):
60115,r=b?Symbol.for("react.lazy"):60116;function t(a){if("object"===typeof a&&null!==a){var u=a.$$typeof;switch(u){case c:switch(a=a.type,a){case l:case m:case e:case g:case f:case p:return a;default:switch(a=a&&a.$$typeof,a){case k:case n:case h:return a;default:return u}}case r:case q:case d:return u}}}function v(a){return t(a)===m}exports.typeOf=t;exports.AsyncMode=l;exports.ConcurrentMode=m;exports.ContextConsumer=k;exports.ContextProvider=h;exports.Element=c;exports.ForwardRef=n;
exports.Fragment=e;exports.Lazy=r;exports.Memo=q;exports.Portal=d;exports.Profiler=g;exports.StrictMode=f;exports.Suspense=p;exports.isValidElementType=function(a){return "string"===typeof a||"function"===typeof a||a===e||a===m||a===g||a===f||a===p||"object"===typeof a&&null!==a&&(a.$$typeof===r||a.$$typeof===q||a.$$typeof===h||a.$$typeof===k||a.$$typeof===n)};exports.isAsyncMode=function(a){return v(a)||t(a)===l};exports.isConcurrentMode=v;exports.isContextConsumer=function(a){return t(a)===k};
exports.isContextProvider=function(a){return t(a)===h};exports.isElement=function(a){return "object"===typeof a&&null!==a&&a.$$typeof===c};exports.isForwardRef=function(a){return t(a)===n};exports.isFragment=function(a){return t(a)===e};exports.isLazy=function(a){return t(a)===r};exports.isMemo=function(a){return t(a)===q};exports.isPortal=function(a){return t(a)===d};exports.isProfiler=function(a){return t(a)===g};exports.isStrictMode=function(a){return t(a)===f};
exports.isSuspense=function(a){return t(a)===p};
});

unwrapExports(reactIs_production_min);
var reactIs_production_min_1 = reactIs_production_min.typeOf;
var reactIs_production_min_2 = reactIs_production_min.AsyncMode;
var reactIs_production_min_3 = reactIs_production_min.ConcurrentMode;
var reactIs_production_min_4 = reactIs_production_min.ContextConsumer;
var reactIs_production_min_5 = reactIs_production_min.ContextProvider;
var reactIs_production_min_6 = reactIs_production_min.Element;
var reactIs_production_min_7 = reactIs_production_min.ForwardRef;
var reactIs_production_min_8 = reactIs_production_min.Fragment;
var reactIs_production_min_9 = reactIs_production_min.Lazy;
var reactIs_production_min_10 = reactIs_production_min.Memo;
var reactIs_production_min_11 = reactIs_production_min.Portal;
var reactIs_production_min_12 = reactIs_production_min.Profiler;
var reactIs_production_min_13 = reactIs_production_min.StrictMode;
var reactIs_production_min_14 = reactIs_production_min.Suspense;
var reactIs_production_min_15 = reactIs_production_min.isValidElementType;
var reactIs_production_min_16 = reactIs_production_min.isAsyncMode;
var reactIs_production_min_17 = reactIs_production_min.isConcurrentMode;
var reactIs_production_min_18 = reactIs_production_min.isContextConsumer;
var reactIs_production_min_19 = reactIs_production_min.isContextProvider;
var reactIs_production_min_20 = reactIs_production_min.isElement;
var reactIs_production_min_21 = reactIs_production_min.isForwardRef;
var reactIs_production_min_22 = reactIs_production_min.isFragment;
var reactIs_production_min_23 = reactIs_production_min.isLazy;
var reactIs_production_min_24 = reactIs_production_min.isMemo;
var reactIs_production_min_25 = reactIs_production_min.isPortal;
var reactIs_production_min_26 = reactIs_production_min.isProfiler;
var reactIs_production_min_27 = reactIs_production_min.isStrictMode;
var reactIs_production_min_28 = reactIs_production_min.isSuspense;

var reactIs_development = createCommonjsModule(function (module, exports) {



if (process.env.NODE_ENV !== "production") {
  (function() {

Object.defineProperty(exports, '__esModule', { value: true });

// The Symbol used to tag the ReactElement-like types. If there is no native Symbol
// nor polyfill, then a plain number is used for performance.
var hasSymbol = typeof Symbol === 'function' && Symbol.for;

var REACT_ELEMENT_TYPE = hasSymbol ? Symbol.for('react.element') : 0xeac7;
var REACT_PORTAL_TYPE = hasSymbol ? Symbol.for('react.portal') : 0xeaca;
var REACT_FRAGMENT_TYPE = hasSymbol ? Symbol.for('react.fragment') : 0xeacb;
var REACT_STRICT_MODE_TYPE = hasSymbol ? Symbol.for('react.strict_mode') : 0xeacc;
var REACT_PROFILER_TYPE = hasSymbol ? Symbol.for('react.profiler') : 0xead2;
var REACT_PROVIDER_TYPE = hasSymbol ? Symbol.for('react.provider') : 0xeacd;
var REACT_CONTEXT_TYPE = hasSymbol ? Symbol.for('react.context') : 0xeace;
var REACT_ASYNC_MODE_TYPE = hasSymbol ? Symbol.for('react.async_mode') : 0xeacf;
var REACT_CONCURRENT_MODE_TYPE = hasSymbol ? Symbol.for('react.concurrent_mode') : 0xeacf;
var REACT_FORWARD_REF_TYPE = hasSymbol ? Symbol.for('react.forward_ref') : 0xead0;
var REACT_SUSPENSE_TYPE = hasSymbol ? Symbol.for('react.suspense') : 0xead1;
var REACT_MEMO_TYPE = hasSymbol ? Symbol.for('react.memo') : 0xead3;
var REACT_LAZY_TYPE = hasSymbol ? Symbol.for('react.lazy') : 0xead4;

function isValidElementType(type) {
  return typeof type === 'string' || typeof type === 'function' ||
  // Note: its typeof might be other than 'symbol' or 'number' if it's a polyfill.
  type === REACT_FRAGMENT_TYPE || type === REACT_CONCURRENT_MODE_TYPE || type === REACT_PROFILER_TYPE || type === REACT_STRICT_MODE_TYPE || type === REACT_SUSPENSE_TYPE || typeof type === 'object' && type !== null && (type.$$typeof === REACT_LAZY_TYPE || type.$$typeof === REACT_MEMO_TYPE || type.$$typeof === REACT_PROVIDER_TYPE || type.$$typeof === REACT_CONTEXT_TYPE || type.$$typeof === REACT_FORWARD_REF_TYPE);
}

/**
 * Forked from fbjs/warning:
 * https://github.com/facebook/fbjs/blob/e66ba20ad5be433eb54423f2b097d829324d9de6/packages/fbjs/src/__forks__/warning.js
 *
 * Only change is we use console.warn instead of console.error,
 * and do nothing when 'console' is not supported.
 * This really simplifies the code.
 * ---
 * Similar to invariant but only logs a warning if the condition is not met.
 * This can be used to log issues in development environments in critical
 * paths. Removing the logging code for production environments will keep the
 * same logic and follow the same code paths.
 */

var lowPriorityWarning = function () {};

{
  var printWarning = function (format) {
    for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      args[_key - 1] = arguments[_key];
    }

    var argIndex = 0;
    var message = 'Warning: ' + format.replace(/%s/g, function () {
      return args[argIndex++];
    });
    if (typeof console !== 'undefined') {
      console.warn(message);
    }
    try {
      // --- Welcome to debugging React ---
      // This error was thrown as a convenience so that you can use this stack
      // to find the callsite that caused this warning to fire.
      throw new Error(message);
    } catch (x) {}
  };

  lowPriorityWarning = function (condition, format) {
    if (format === undefined) {
      throw new Error('`lowPriorityWarning(condition, format, ...args)` requires a warning ' + 'message argument');
    }
    if (!condition) {
      for (var _len2 = arguments.length, args = Array(_len2 > 2 ? _len2 - 2 : 0), _key2 = 2; _key2 < _len2; _key2++) {
        args[_key2 - 2] = arguments[_key2];
      }

      printWarning.apply(undefined, [format].concat(args));
    }
  };
}

var lowPriorityWarning$1 = lowPriorityWarning;

function typeOf(object) {
  if (typeof object === 'object' && object !== null) {
    var $$typeof = object.$$typeof;
    switch ($$typeof) {
      case REACT_ELEMENT_TYPE:
        var type = object.type;

        switch (type) {
          case REACT_ASYNC_MODE_TYPE:
          case REACT_CONCURRENT_MODE_TYPE:
          case REACT_FRAGMENT_TYPE:
          case REACT_PROFILER_TYPE:
          case REACT_STRICT_MODE_TYPE:
          case REACT_SUSPENSE_TYPE:
            return type;
          default:
            var $$typeofType = type && type.$$typeof;

            switch ($$typeofType) {
              case REACT_CONTEXT_TYPE:
              case REACT_FORWARD_REF_TYPE:
              case REACT_PROVIDER_TYPE:
                return $$typeofType;
              default:
                return $$typeof;
            }
        }
      case REACT_LAZY_TYPE:
      case REACT_MEMO_TYPE:
      case REACT_PORTAL_TYPE:
        return $$typeof;
    }
  }

  return undefined;
}

// AsyncMode is deprecated along with isAsyncMode
var AsyncMode = REACT_ASYNC_MODE_TYPE;
var ConcurrentMode = REACT_CONCURRENT_MODE_TYPE;
var ContextConsumer = REACT_CONTEXT_TYPE;
var ContextProvider = REACT_PROVIDER_TYPE;
var Element = REACT_ELEMENT_TYPE;
var ForwardRef = REACT_FORWARD_REF_TYPE;
var Fragment = REACT_FRAGMENT_TYPE;
var Lazy = REACT_LAZY_TYPE;
var Memo = REACT_MEMO_TYPE;
var Portal = REACT_PORTAL_TYPE;
var Profiler = REACT_PROFILER_TYPE;
var StrictMode = REACT_STRICT_MODE_TYPE;
var Suspense = REACT_SUSPENSE_TYPE;

var hasWarnedAboutDeprecatedIsAsyncMode = false;

// AsyncMode should be deprecated
function isAsyncMode(object) {
  {
    if (!hasWarnedAboutDeprecatedIsAsyncMode) {
      hasWarnedAboutDeprecatedIsAsyncMode = true;
      lowPriorityWarning$1(false, 'The ReactIs.isAsyncMode() alias has been deprecated, ' + 'and will be removed in React 17+. Update your code to use ' + 'ReactIs.isConcurrentMode() instead. It has the exact same API.');
    }
  }
  return isConcurrentMode(object) || typeOf(object) === REACT_ASYNC_MODE_TYPE;
}
function isConcurrentMode(object) {
  return typeOf(object) === REACT_CONCURRENT_MODE_TYPE;
}
function isContextConsumer(object) {
  return typeOf(object) === REACT_CONTEXT_TYPE;
}
function isContextProvider(object) {
  return typeOf(object) === REACT_PROVIDER_TYPE;
}
function isElement(object) {
  return typeof object === 'object' && object !== null && object.$$typeof === REACT_ELEMENT_TYPE;
}
function isForwardRef(object) {
  return typeOf(object) === REACT_FORWARD_REF_TYPE;
}
function isFragment(object) {
  return typeOf(object) === REACT_FRAGMENT_TYPE;
}
function isLazy(object) {
  return typeOf(object) === REACT_LAZY_TYPE;
}
function isMemo(object) {
  return typeOf(object) === REACT_MEMO_TYPE;
}
function isPortal(object) {
  return typeOf(object) === REACT_PORTAL_TYPE;
}
function isProfiler(object) {
  return typeOf(object) === REACT_PROFILER_TYPE;
}
function isStrictMode(object) {
  return typeOf(object) === REACT_STRICT_MODE_TYPE;
}
function isSuspense(object) {
  return typeOf(object) === REACT_SUSPENSE_TYPE;
}

exports.typeOf = typeOf;
exports.AsyncMode = AsyncMode;
exports.ConcurrentMode = ConcurrentMode;
exports.ContextConsumer = ContextConsumer;
exports.ContextProvider = ContextProvider;
exports.Element = Element;
exports.ForwardRef = ForwardRef;
exports.Fragment = Fragment;
exports.Lazy = Lazy;
exports.Memo = Memo;
exports.Portal = Portal;
exports.Profiler = Profiler;
exports.StrictMode = StrictMode;
exports.Suspense = Suspense;
exports.isValidElementType = isValidElementType;
exports.isAsyncMode = isAsyncMode;
exports.isConcurrentMode = isConcurrentMode;
exports.isContextConsumer = isContextConsumer;
exports.isContextProvider = isContextProvider;
exports.isElement = isElement;
exports.isForwardRef = isForwardRef;
exports.isFragment = isFragment;
exports.isLazy = isLazy;
exports.isMemo = isMemo;
exports.isPortal = isPortal;
exports.isProfiler = isProfiler;
exports.isStrictMode = isStrictMode;
exports.isSuspense = isSuspense;
  })();
}
});

unwrapExports(reactIs_development);
var reactIs_development_1 = reactIs_development.typeOf;
var reactIs_development_2 = reactIs_development.AsyncMode;
var reactIs_development_3 = reactIs_development.ConcurrentMode;
var reactIs_development_4 = reactIs_development.ContextConsumer;
var reactIs_development_5 = reactIs_development.ContextProvider;
var reactIs_development_6 = reactIs_development.Element;
var reactIs_development_7 = reactIs_development.ForwardRef;
var reactIs_development_8 = reactIs_development.Fragment;
var reactIs_development_9 = reactIs_development.Lazy;
var reactIs_development_10 = reactIs_development.Memo;
var reactIs_development_11 = reactIs_development.Portal;
var reactIs_development_12 = reactIs_development.Profiler;
var reactIs_development_13 = reactIs_development.StrictMode;
var reactIs_development_14 = reactIs_development.Suspense;
var reactIs_development_15 = reactIs_development.isValidElementType;
var reactIs_development_16 = reactIs_development.isAsyncMode;
var reactIs_development_17 = reactIs_development.isConcurrentMode;
var reactIs_development_18 = reactIs_development.isContextConsumer;
var reactIs_development_19 = reactIs_development.isContextProvider;
var reactIs_development_20 = reactIs_development.isElement;
var reactIs_development_21 = reactIs_development.isForwardRef;
var reactIs_development_22 = reactIs_development.isFragment;
var reactIs_development_23 = reactIs_development.isLazy;
var reactIs_development_24 = reactIs_development.isMemo;
var reactIs_development_25 = reactIs_development.isPortal;
var reactIs_development_26 = reactIs_development.isProfiler;
var reactIs_development_27 = reactIs_development.isStrictMode;
var reactIs_development_28 = reactIs_development.isSuspense;

var reactIs = createCommonjsModule(function (module) {

if (process.env.NODE_ENV === 'production') {
  module.exports = reactIs_production_min;
} else {
  module.exports = reactIs_development;
}
});

/**
 * Copyright 2015, Yahoo! Inc.
 * Copyrights licensed under the New BSD License. See the accompanying LICENSE file for terms.
 */

var REACT_STATICS = {
    childContextTypes: true,
    contextType: true,
    contextTypes: true,
    defaultProps: true,
    displayName: true,
    getDefaultProps: true,
    getDerivedStateFromError: true,
    getDerivedStateFromProps: true,
    mixins: true,
    propTypes: true,
    type: true
};

var KNOWN_STATICS = {
    name: true,
    length: true,
    prototype: true,
    caller: true,
    callee: true,
    arguments: true,
    arity: true
};

var FORWARD_REF_STATICS = {
    '$$typeof': true,
    render: true,
    defaultProps: true,
    displayName: true,
    propTypes: true
};

var MEMO_STATICS = {
    '$$typeof': true,
    compare: true,
    defaultProps: true,
    displayName: true,
    propTypes: true,
    type: true
};

var TYPE_STATICS = {};
TYPE_STATICS[reactIs.ForwardRef] = FORWARD_REF_STATICS;

function getStatics(component) {
    if (reactIs.isMemo(component)) {
        return MEMO_STATICS;
    }
    return TYPE_STATICS[component['$$typeof']] || REACT_STATICS;
}

var defineProperty = Object.defineProperty;
var getOwnPropertyNames = Object.getOwnPropertyNames;
var getOwnPropertySymbols = Object.getOwnPropertySymbols;
var getOwnPropertyDescriptor = Object.getOwnPropertyDescriptor;
var getPrototypeOf = Object.getPrototypeOf;
var objectPrototype = Object.prototype;

function hoistNonReactStatics(targetComponent, sourceComponent, blacklist) {
    if (typeof sourceComponent !== 'string') {
        // don't hoist over string (html) components

        if (objectPrototype) {
            var inheritedComponent = getPrototypeOf(sourceComponent);
            if (inheritedComponent && inheritedComponent !== objectPrototype) {
                hoistNonReactStatics(targetComponent, inheritedComponent, blacklist);
            }
        }

        var keys = getOwnPropertyNames(sourceComponent);

        if (getOwnPropertySymbols) {
            keys = keys.concat(getOwnPropertySymbols(sourceComponent));
        }

        var targetStatics = getStatics(targetComponent);
        var sourceStatics = getStatics(sourceComponent);

        for (var i = 0; i < keys.length; ++i) {
            var key = keys[i];
            if (!KNOWN_STATICS[key] && !(blacklist && blacklist[key]) && !(sourceStatics && sourceStatics[key]) && !(targetStatics && targetStatics[key])) {
                var descriptor = getOwnPropertyDescriptor(sourceComponent, key);
                try {
                    // Avoid failures from read-only properties
                    defineProperty(targetComponent, key, descriptor);
                } catch (e) {}
            }
        }

        return targetComponent;
    }

    return targetComponent;
}

var hoistNonReactStatics_cjs = hoistNonReactStatics;

function withLocalize(WrappedComponent) {
    var LocalizedComponent = /** @class */ (function (_super) {
        __extends(LocalizedComponent, _super);
        function LocalizedComponent() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        LocalizedComponent.prototype.render = function () {
            var _this = this;
            return (React.createElement(LocalizeContext.Consumer, null, function (context) { return React.createElement(WrappedComponent, __assign({}, context, _this.props)); }));
        };
        return LocalizedComponent;
    }(Component));
    hoistNonReactStatics_cjs(LocalizedComponent, WrappedComponent);
    return LocalizedComponent;
}

var LocalizeProvider = function (props) {
    var initExternalStore = function () {
        var store = props.store, initialize$1 = props.initialize;
        if (store && initialize$1) {
            store.dispatch(initialize(initialize$1));
        }
    };
    var initialState = props.initialize !== undefined
        ? localizeReducer(undefined, {
            type: INITIALIZE,
            payload: props.initialize
        })
        : localizeReducer(undefined, {});
    var _a = useState(localizeReducer(initialState, {})), localize = _a[0], setLocalize = _a[1];
    var onStateDidChange = function (prevState) {
        if (!props.store) {
            return;
        }
        var getState = props.getState || (function (state) { return state.localize; });
        var prevLocalizeState = prevState && getState(prevState);
        var curLocalizeState = getState(props.store.getState());
        var prevActiveLanguage = prevState && getActiveLanguage(prevLocalizeState);
        var curActiveLanguage = getActiveLanguage(curLocalizeState);
        var prevOptions = prevState && getOptions(prevLocalizeState);
        var curOptions = getOptions(curLocalizeState);
        var prevTranslations = prevState && getTranslationsForActiveLanguage(prevLocalizeState);
        var curTranslations = getTranslationsForActiveLanguage(curLocalizeState);
        var hasActiveLangaugeChanged = (prevActiveLanguage && prevActiveLanguage.code) !==
            (curActiveLanguage && curActiveLanguage.code);
        var hasOptionsChanged = prevOptions !== curOptions;
        var hasTranslationsChanged = prevTranslations !== curTranslations;
        if (hasActiveLangaugeChanged ||
            hasOptionsChanged ||
            hasTranslationsChanged) {
            setLocalize(curLocalizeState);
        }
    };
    useEffect(function () {
        initExternalStore();
        var unsubscribeFromStore = subscribeToExternalStore();
        return function () { return unsubscribeFromStore && unsubscribeFromStore(); };
    }, []);
    var subscribeToExternalStore = function () {
        if (props.store) {
            return storeDidChange(props.store, onStateDidChange);
        }
    };
    var dispatch = props.store
        ? props.store.dispatch
        : function (action) { return setLocalize(function (prevState) { return localizeReducer(prevState, action); }); };
    var getContextPropsSelector = getContextPropsFromState(dispatch);
    var contextProps = getContextPropsSelector(localize);
    return (React.createElement(LocalizeContext.Provider, { value: contextProps }, props.children));
};

export { LocalizeContext, LocalizeProvider, Translate, addTranslation, addTranslationForLanguage, getActiveLanguage, getLanguages, getOptions, getTranslate, getTranslations, initialize, localizeReducer, setActiveLanguage, withLocalize };
//# sourceMappingURL=index.js.map
